..
.. A new scriv changelog fragment
..
.. Uncomment the header that is right (remove the leading dots).
..
.. Leave the "(:pr:`...`)" text in your change description.
.. GitHub Actions will automatically replace it when the PR is merged.
..
{% for cat in config.categories -%}
.. {{ cat }}
.. {{ config.rst_header_chars[1] * (cat|length) }}
..
.. - A bullet item for the {{ cat }} category. (:pr:`NUMBER`)
..
{% endfor -%}
