from globus_sdk.testing.models import RegisteredResponse, ResponseSet

from .._common import ENDPOINT_ID

RESPONSES = ResponseSet(
    metadata={"endpoint_id": ENDPOINT_ID},
    default=RegisteredResponse(
        service="compute",
        path=f"/v2/endpoints/{ENDPOINT_ID}",
        method="DELETE",
        json={"result": 302},
    ),
)
